﻿using HardwareShopContracts.BindingModels;
using HardwareShopContracts.SearchModels;
using HardwareShopContracts.ViewModels;
using HardwareShopDatabaseImplement;
using HardwareShopDatabaseImplement.Models.Storekeeper;
using HardwareShopDatabaseImplement.Models.Worker;
using Microsoft.EntityFrameworkCore;

namespace HardwareShopContracts.StoragesContracts
{
    public class PurchaseStorage : IPurchaseStorage
    {
        public List<PurchaseViewModel> GetFullList()
        {
            using var context = new HardwareShopDatabase();
            return context.Purchases
                    .Include(x => x.Builds)
                    .Include(x => x.Goods)
                    .Include(x => x.User)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<PurchaseViewModel> GetFilteredList(PurchaseSearchModel model)
        {
            using var context = new HardwareShopDatabase();
            if (model.UserId.HasValue)
            {
                return context.Purchases
                    .Include(x => x.Builds)
                    .Include(x => x.Goods)
                    .Include(x => x.User)
                    .Where(x => x.UserId == model.UserId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return context.Purchases
                    .Include(x => x.Builds)
                    .Include(x => x.Goods)
                    .Include(x => x.User)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public PurchaseViewModel? GetElement(PurchaseSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new HardwareShopDatabase();
            return context.Purchases
                .Include(x => x.Builds)
                .Include(x => x.Goods)
                .Include(x => x.User)
                .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
                ?.GetViewModel;
        }

        public PurchaseViewModel? Insert(PurchaseBindingModel model)
        {
            using var context = new HardwareShopDatabase();
            var newPurchase = Purchase.Create(context, model);
            if (newPurchase == null)
            {
                return null;
            }
            context.Purchases.Add(newPurchase);
            context.SaveChanges();
            return context.Purchases
                .Include(x => x.Builds)
                .Include(x => x.Goods)
                .Include(x => x.User)
                .FirstOrDefault(x => x.Id == newPurchase.Id)
                 ?.GetViewModel;
        }

        public PurchaseViewModel? Update(PurchaseBindingModel model)
        {
            using var context = new HardwareShopDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var purchase = context.Purchases
                    .Include(x => x.Builds)
                    .Include(x => x.Goods)
                    .Include(x => x.User)
                    .FirstOrDefault(x => x.Id == model.Id);
                if (purchase == null)
                {
                    return null;
                }
                purchase.Update(model);
                context.SaveChanges();
                purchase.UpdateGoods(context, model);
                purchase.UpdateBuilds(context, model);
                transaction.Commit();
                return purchase.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }

        public PurchaseViewModel? Delete(PurchaseBindingModel model)
        {
            using var context = new HardwareShopDatabase();
            var element = context.Purchases
                .Include(x => x.Builds)
                .Include(x => x.Goods)
                .Include(x => x.User)
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Purchases.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
